@@ -390,6 +390,11 @@ catalog_png_exporter_init (CatalogPngExporter *ce)
 	font_desc = pango_font_description_from_string (DEFAULT_FONT);
 	pango_layout_set_font_description (ce->layout, font_desc);
 	pango_font_description_free (font_desc); 
+
+	/**/
+
+	ce->exporting = FALSE;
+	ce->interrupted = FALSE;
 }
 
 
@@ -846,6 +851,16 @@ export (CatalogPngExporter *ce)
 	do {
 		int row_height;
 
+		if (ce->interrupted) {
+			if (ce->file_list != NULL) {
+				g_list_foreach (ce->file_list, (GFunc) image_data_free, NULL);
+				g_list_free (ce->file_list);
+				ce->file_list = NULL;
+			}
+			g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[DONE], 0);
+			goto label_end;
+		}
+ 
 		/* get items to paint. */
 
 		row_first_item = scan_file;
@@ -1260,6 +1275,16 @@ load_next_file (CatalogPngExporter *ce)
 	char *filename;
 	char *utf8_name;
 
+	if (ce->interrupted) {
+		if (ce->file_list != NULL) {
+			g_list_foreach (ce->file_list, (GFunc) image_data_free, NULL);
+			g_list_free (ce->file_list);
+			ce->file_list = NULL;
+		}
+		g_signal_emit (G_OBJECT (ce), catalog_png_exporter_signals[DONE], 0);
+		return;
+	}
+
 	g_signal_emit (G_OBJECT (ce), 
 		       catalog_png_exporter_signals[PROGRESS],
 		       0,
@@ -1351,6 +1376,10 @@ catalog_png_exporter_export (CatalogPngExporter *ce)
 	g_return_if_fail (ce->thumb_width != 0);
 	g_return_if_fail (ce->thumb_height != 0);
 
+	if (ce->exporting)
+		return;
+	ce->exporting = TRUE;
+
 	if (ce->tloader != NULL)
 		g_object_unref (G_OBJECT (ce->tloader));
 
@@ -1380,6 +1409,17 @@ catalog_png_exporter_export (CatalogPngExporter *ce)
 }
 
 
+void
+catalog_png_exporter_interrupt (CatalogPngExporter *ce)
+{
+	g_return_if_fail (IS_CATALOG_PNG_EXPORTER (ce));
+
+	if (! ce->exporting)
+		return;
+	ce->interrupted = TRUE;
+}
+
+
 
 /* ----- */
 
@@ -1777,12 +1817,11 @@ static void
 paint_comment (CatalogPngExporter *ce,
 	       int                 x,
 	       int                 y,
-	       char               *text,
+	       char               *utf8_text,
 	       int                *height)
 {
 	PangoRectangle        bounds;
 	const char           *font_name;
-	char                 *utf8_text;
 	char                 *escaped_text;
 	char                 *marked_text;
 	char                 *parsed_text;
@@ -1803,11 +1842,9 @@ paint_comment (CatalogPngExporter *ce,
 		pango_attr_list_ref (original_attr_list);
 	x += FRAME_BORDER;
 
-	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
 	escaped_text = g_markup_escape_text (utf8_text, -1);
 	marked_text = g_strdup_printf ("<i>%s</i>", escaped_text);
 	g_free (escaped_text);
-	g_free (utf8_text);
 
 	if (! pango_parse_markup (marked_text, -1,
 				  0,
@@ -1815,7 +1852,7 @@ paint_comment (CatalogPngExporter *ce,
 				  &parsed_text, 
 				  NULL,
 				  &error)) {
-		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, text);
+		g_warning ("Failed to set text from markup due to error parsing markup: %s\nThis is the text that caused the error: %s",  error->message, utf8_text);
 		g_error_free (error);
 		g_free (marked_text);
 		pango_attr_list_unref (original_attr_list);
